<?php
/**
 * @copyright © 2007, 2008, 2009 Intrahealth International, Inc.
 * This File is part of I2CE
 *
 * I2CE is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @package I2CE
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @since v1.0.0
 * @version v2.0.0
 */
/**
 * This factory is used to create instances of I2CE_Form objects from the form name.
 * @package I2CE 
 * @access public
 */
class I2CE_FormFactory extends I2CE_Fuzzy {
    /**
     * @var I2CE_FormFactory The single instance of this class.
     */
    static private $instance;
    /**
     * @var array The classes that have been registered with the factory.
     */
    private $classes;
        
    /**
     * Return the instance of this factory and create it if it doesn't exist.
     */
    static public function instance() {
        if ( ! self::$instance instanceof I2CE_FormFactory ) {
            self::$instance = new I2CE_FormFactory();
        }
        return self::$instance;
    }


        
    public function __construct() {
        $this->classes = I2CE::getConfig()->modules->forms->forms;
        $this->classHierarchy= array();
        $forms = $this->classes->getKeys();
    }
        
    /**
     * Checks to see if the given form has been registered.
     * @param string $form The name of the form.
     * @return boolean
     */
    public function exists( $form ) {
        return isset($this->classes->$form);
    }
        
    /**
     * get the available forms.
     *@returns array with values the form name
     */
    public function getForms() {
        return $this->classes->getKeys();
    }





    /**
     * Get the class hierarchy associated to a class.
     * @returns array The keys of the array range from 0 to N.  The value of key 0 is the class associated to the form.  The last key is has value 'I2CE_Form'.  
     * Returns null in error.
     */
    public  function getClassHierarchy($form) {
        if ( !$this->exists( $form ) ) {
            //I2CE::raiseError( "Unknown form: $form");
            return null;
        }
        if (!array_key_exists($form,$this->classHierarchy) || !is_array($this->classHierarchy[$form])) {
            $this->classHierarchy[$form] = array();
            $class = null;
            $this->classes->setIfIsSet($class,$form . '/class');
            $formClass = I2CE::getConfig()->modules->forms->formClasses;
            while ((!empty($class)) && ($class != 'I2CE_Fuzzy')) {
                if (!isset($formClass->$class)) {
                    I2CE::raiseError("Could not find the form $form's class $class information in it's class hierarchy");
                    return null;
                }
                $this->classHierarchy[$form][] = $class;
                if (!isset($formClass->$class->extends)) {
                    I2CE::raiseError("Class extension information is absent for $class");
                    return null;
                }
                $class = $formClass->$class->extends;
            }
        }
        return $this->classHierarchy[$form];
    }



    public function getClassName($form) {
        if (I2CE::getConfig()->is_scalar("/modules/forms/forms/$form/class")) {
            return I2CE::getConfig()->traverse("/modules/forms/forms/$form/class");
        } else {
            return false;
        }
    }
        
    /**
     * Return an instance of a I2CE_Form from the factory.
     * @param string $formId The name of the form possible with an id
     * @return I2CE_Form or null on failure
     */
    public function createForm( $formId) {
        if (is_array($formId) && count($formId) == 2) {
            list($form,$id) = $formId;
        } else if (is_string($formId)) {
            if (strpos($formId,'|') === false) { //no id
                $form = $formId;
                $id = '0';
            } else {
                list($form,$id) = explode('|',$formId,2);
                if (strlen($id) == 0) {
                    $id = '0';
                }
            }
        } else {
            return null;
        }
        $class = $this->getClassName($form);
        if ($class === false) {
            return null;
        }
        $obj = new $class( $form, $id );
        if (!($obj instanceof I2CE_Form)) {
            I2CE::raiseError( "$form is not a subclass of I2CE_Form");
            return null;
        }
        if (!empty($this->classes->$form->display)) {
            $obj->setDisplayName( $this->classes->$form->display );
        } else {
            $obj->setDisplayName( $this->classes->$form->class );
        }
        return $obj;
    }
        
    /**
     * Call a static function in the form's class object.
     * @param string $form The name of the form.
     * @param string $func The function to be called.
     * @param array $args The arguments to pass to the function.
     */
    public function callStatic( $form, $func, $args=array() ) {
        $class = $this->getClassName($form);
        if ($class !== false) {
            if ($args !== null) {
                return call_user_func_array( array( $class, $func ), $args );                 
            } else {
                return call_user_func( array( $class, $func ) );                 
            }
        }
        //I2CE::raiseError( "callStatic: Unknown form: $form");
        return null;
    } 
        
    /**
     * Return a static variable from the form's class object.
     * @param string $form The name of the form.
     * @param string $var The name of the variable.
     * @param boolean $const Set to true to get a constant instead of a variable
     */
    public function getStatic( $form, $var, $const=false ) {
        $class = $this->getClassName($form);
        if ($class === false) {
            return null;
        }
        $val = null;
        eval( '$val = ' . $class . '::' . ( !$const ? '$' : '' ) . $var . ';' );
        return $val;
    }

    /**
     * Get the fields for a given form via what is stored in magic data.
     * @param string $form the short name of a form
     * @param mixed  $restict. Defaults to null which means there are no restrictions. Otherwise it should
     * be an array consisting of any numbder of the following keys 'in_db', 'required', and 'unique'  the values
     * associated to each key should be true or false.  We then only return fields who match the given value
     * @returns associative array of string.  key is the field the value is the form class
     */
    public function getFieldNames($form,$restrictions = array()){
        $fields = array();
        $class = '';
        I2CE::getConfig()->setIfIsSet($class,"/modules/forms/forms/$form/class");
        if (empty($class)) {
            return $fields;
        }
        $formClassConfig = I2CE::getConfig()->modules->forms->formClasses;
        $classes = $this->getClassHierarchy($form);
        $num = count($classes);
        $defaults = array('in_db'=>true,'required'=>false,'unique'=>false);
        if (!is_array($restrictions)) {
            $restrictions = array();
        }
        if ( !is_array( $classes ) ) {
            return $fields;
        }
        foreach ($classes as $class) {
            $t_fields = array();
            if (isset($formClassConfig->$class->fields)) {
                $t_fields = $formClassConfig->$class->fields->getKeys();
            }
            $fieldConfigBase = $formClassConfig->$class->fields;
            foreach ($t_fields as $field) {
                $fieldConfig = $fieldConfigBase->$field;
                $add = true;
                foreach ($restrictions as $restriction=>$desired) {
                    if (!array_key_exists($restriction,$defaults)) {
                        continue;
                    }
                    $status = $defaults[$restriction];
                    $fieldConfig->setIfIsSet($status,$restriction);
                    if ($status != $desired) {
                        $add = false;
                    }
                }
                if ($add) {
                    $fields[$field] = $class;                
                }
            }
        }
        return $fields;
    }

    

    /**
     * Shortcut to retrieving a constant value from a form's class object.
     * @param string $form The name of the form.
     * @param string $var The name of the constant.
     */
    public function getConst( $form, $var ) {
        return $this->getStatic( $form, $var, true );
    }
        

}

# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
