<?php
/**
 * @copyright © 2007, 2008, 2009 Intrahealth International, Inc.
 * This File is part of I2CE
 *
 * I2CE is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
    * @package I2CE
    * @author Carl Leitner <litlfred@ibiblio.org>
    * @since v2.0.0
    * @version v2.0.0
    */
/**
 * Class for defining all the database fields used by a {@link I2CE_Form} object.
 * @package I2CE
 * @access public
 */
abstract class I2CE_FormField_BINARY_FILE extends I2CE_FormField_DB_BLOB {
    /**
     * The magically determined mime type
     * @var protected mixed $file_name.  Either false or a string
     */
    protected $mime_type = false;    


    /**
     * The file name as it was uploaded (may not be set)
     * @var protected mixed $file_name.  Either false or string
     */
    protected $file_name = false;    

    /**
     * The time the file  was uploaded (may not be set)
     * @var protected mixed $fmod_time.  Either false or integer(unix time stamp)
     */
    protected $fmod_time = false;    


    /**
     * Gets the binary data for the form field
     * @returns string
     */
    public function getBinaryData() {
        return $this->value;
    }

    /**
     * Create a new instance of a I2CE_FormField
     * @param string $name
     * @param array $options A list of options for this form field.
     */
    public function __construct( $name, $options=array() ) {
        parent::__construct($name, $options);
    }

    /**
     *  An array whose keys are the keys to the metadata for the file and values are the variables
     * they should be stored in .  All keys need to be a string of length 9
     * @protected static array of string $keys
     */
    protected static $keys = array('mime-type'=>'mime_type','file-name'=>'file_name' , 'fmod-time'=>'fmod_time');

    /**
     * Sets the value of this field from the database format.
     * @param mixed $value
     */
    public function setFromDB( $value ) {
        //keys should be fixed length of 9 so we can reduce the string processing.
        //format is mime-type<VAL1>file-name<val-2>some-keys<VAL3>datadatadatadata
        //keys don't need to be in a fixed order
        while ( $value[9] == '<' ) {
            $key = substr($value,0,9);
            if (!array_key_exists($key, self::$keys) || ($pos=strpos($value,'>')) === false) { 
                break;
            }
            $var = self::$keys[$key];
            $this->$var = substr($value,10,$pos - 10);
            $value = substr($value,$pos + 1); 
        }
        parent::setFromDB($value);
    }


    /** 
     * Returns the value of this field ready to be stored in the database.
     * @return mixed
     */
    public function getDBValue() {
        $ret = '';
        foreach (self::$keys as $key=>$var) {
            if ($this->$var) {
                $ret .= $key . '<' . $this->$var . '>';
            }
        }
        return $ret . $this->value;
    }


    /**
     * Returns the value of this field as a human readable format.
     * @param I2CE_Entry $entry If an I2CE_Entry object is passed then it will return the value
     * for that entry assuming it's an entry for this field.
     * @return mixed
     */
    public function getDisplayValue( $entry = false ) {
        if( $entry instanceof I2CE_Entry ) {
            return $entry->getMimeType() . ' File Size: ' . strlen( $entry->getValue() );
        } else {
            return $this->getMimeType() . ' File Size: ' . strlen( $this->value );
        }
    }

    /**
     * Gets the length of the conten
     * @returns int
     */
    public function getContentLength() {
        return strlen($this->value);
    }

    /**
     * Returns the last modification time of this field, or false if unknown
     * @param mixed.
     */
    public function getModTime() {
        return $this->fmod_time;
    }

    /**
     * Generate the link to the given binary field in the given form id
     * @param string $formid
     * @param string $field
     * @returns string
     */
    public static function  getFieldLink($formid,$field) {
        return  "BinField?formid=" . urlencode($formid) .  '&field=' . urlencode($field);
    }

    /**
     *  Gets the link to this binary field.  It will fail if there is field is not attached to any form
     *  @return mixed string on success or null on failure.
     */
    protected function getLink() {
        if (!$this->form instanceof I2CE_Form) {
            return null;
        }
        return self::getFieldLink($this->form->getFormId(),$this->name);
    }


    public function setFromData($data,$file_name,$mime_type=false, $fmod_time = false) {
        $this->value = $data;
        $this->file_name = $file_name;
        if ($fmod_time == false) {
            $this->fmod_time = time();
        } else {
            $fmod_time = $fmod_time;
        }
        $this->mime_type = $mime_type;
        if (!$this->mime_type || $this->mime_type == 'application/unknown' || $this->mime_type == 'application/x-download') {
            $this->mime_type = I2CE_MimeTypes::magicMimeType($this->value);
        }
        if (!$this->isValidMimeType($this->mime_type)) {
            //let's try to get it from the extension of the uploaded file
            if ( ($pos = strrpos($file_name, '.')) !== false) {
                $ext = substr($file_name, $pos + 1);
                $this->mime_type = I2CE_MimeTypes::extToMimeType($ext);
                if (!$this->isValidMimeType($this->mime_type)) {
                    I2CE::raiseError("Unable to upload file $file_name, Invalid mime type from extension");
                    $this->file_name = false;
                    $this->fmod_time = false;
                    $this->mime_type = false;
                    $this->value = false;
                    return;
                }
            } else {
                I2CE::raiseError("Unable to set file $file_name, Invalid mime type");
                $this->file_name = false;
                $this->fmod_time = false;
                $this->mime_type = false;
                $this->value = false;
                return;
            }
        }

    }



    /** 
     * Sets the value of this field from the posted form.
     * @param mixed $post
     */
    public function setFromPost( $post ) {
        $file_indx = $this->getHTMLName() . ':file';
        if (!array_key_exists($file_indx,$_FILES) || !is_array($_FILES[$file_indx])|| !array_key_exists('tmp_name',$_FILES[$file_indx])){
            I2CE::raiseError("No file: $file_indx:" . print_r($_FILES,true));
            return;
        }
        //for error codes see:http://php.net/manual/en/features.file-upload.errors.php
        // if ($_FILES[$file_indx]['error'] != UPLOAD_ERR_OK) {
        //     I2CE::raiseError("Error uploading file: $file_indx:" . print_r($_FILES,true));
        //     return;
        // }
        $file = $_FILES[$file_indx]['tmp_name'];
        if (!file_exists($file)) {
            I2CE::raiseError("Unable to upload file {$_FILES[$file_indx]['name']}, File too large?");
            return;
        }
        $data = '';
        $fh = fopen ($file,'rb');
        while (!feof($fh)) {
            $data .= fread($fh,8192);
        }
        fclose($fh);
        if (strlen($data) != $_FILES[$file_indx]['size']) {
            //an error occured somehow
            I2CE::raiseError('Unable to read uploaded file ' . $_FILES[$file_indx]['name'] . "\n" . strlen($data) . '/' . $_FILES[$file_indx]['size']);
            $data = null;
            return;
        }
        return $this->setFromData($data,$_FILES[$file_indx]['name'],$_FILES[$file_indx]['type']);
    } 



    /**
     * Return the display value of this form field as a DOM Node.
     * @param DOMNode $node  
     * @param I2CE_Template $template
     * @return DOMNode
     */
    public function getDisplayNode( $node, $template ) {
        if ($this->form instanceof I2CE_Form) {
            $link_node = $template->createElement(
                'a',
                array("href"=>$this->getLink()),
                $this->getDisplayValue());
            return $link_node;
        } else {
            return $template->createTextNode($this->getDisplayValue());
        }
    }


        
    /**
     * Checks to see if a mime type is a valid  mime type for this binary file
     * @param string $mime_type
     * @returns true if valid.  false otherwise
     */
    abstract public function isValidMimeType($mime_type);

        
    /**
     * Checks to see if the current value for this is set and valid.
     * @return boolean
     */
    public function isValid() {
        if (!$this->isValidMimeType($this->getMimeType())) {
            $this->value = null;
            return false;
        }
        return true;
    }
        

    public function processDOMEditable($node,$template,$form_node) {
        $ele_name = $this->getHTMLName();
        $node->appendChild($template->createElement( "input", array( "name" => $ele_name . ':file' , "type"=>"file", 'size' => 40)));
        //we need to add a hidden input  node so that $this->setFromPost() is triggeredd  from $form->setFromPost()
        $node->appendChild($template->createElement( "input", array( "name" => $ele_name . ':exists' , "type"=>"hidden", 'value' => 1)));
    }




    /**
     * @returns array of DOMNodes
     */
    public function processDOMNotEditable( $node, $template, $form_node) {
        return array(); //does nothing.
    }



    /**
     *Get the file name associated to this binary field.  if none, it will generate one based on the form and field
     * @returns string
     */
    public function getFileName() {
        if ($this->file_name) {
            $filename  = $this->file_name;
        } else {        
            //no file name is stored, so base it on the form and field name
            if ($this->form instanceof I2CE_Form) {
                $filename = $this->form->form() . '+' . $this->name;
            } else{
                $filename = $this->name;
            }        
            $filename = '.' . $this->getExtension();
        }
        return $filename;

    }
    /**
     *  Any headers that need to be sent when this field is dumped
     * @returns array of string
     */
    public function getHeaders() {
        $headers  = array();
        $headers[] = "Content-Type: " . $this->getMimeType();
        $filename = $this->getFileName();
        $headers[] = "Content-disposition: inline; filename=\"". $filename ."\"";
        return $headers;
    }

    /**
     *get the default extension for this 
     *@returns string
     */
    abstract protected function defaultExtension();


    /**
     *get the default extension for this 
     *@returns string
     */
    abstract protected function defaultMimeType();





    /**
     * Get the magically determined mime type
     * @returns string
     */
    public function getMimeType() {
        if ($this->mime_type === false) { //the mime type was not determined yet
            $this->mime_type = I2CE_MimeTypes::magicMimeType($this->value);            
        }
        if (!$this->mime_type) {
            $this->mime_type = $this->defaultMimeType();
        }
        return $this->mime_type;
    }




    /**
     * Get the extension associated with this binary file
     */
    public function getExtension() {
        $mime_type = $this->getMimeType();
        if (!$mime_type) {
            $mime_type = $this->defaultMimeType();
        }
        return I2CE_MimeTypes::mimeTypeToExt($mime_type);
    }

                



}

# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
