<?php
/**
* © Copyright 2009 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package i2ce
* @subpackage forms
* @author Luke Duncan <lduncan@intrahealth.org>
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v4.0.0
* @since v4.0.0
* @filesource 
*/ 
/** 
* Class I2CE_FormStorage_SDMXHD
* Storage mechanism for reading SDMX-HD code lists.
* 
* @access public
*/


class I2CE_FormStorage_SDMXHD extends   I2CE_FormStorage_Mechanism{

    /**
     * Construct this module class
     * @param string $name The name of this storage mechanism
     * @param array $options
     */
    public function __construct( $name, $options=array() ) {
        parent::__construct( $name, $options );

        if (array_key_exists('closeFile',$options)) {
            $this->closeFile = $options['closeFile'];
        }
    }

    /**
     * whether or not to close the file pointer between access.
     * @var protected boolean $closeFile 
     */
    protected $closeFile = false;


    /**
     * @var protected array $dom_data.  An array indexed by form names of 
     * arrays.  array( 'doc' => DOMDocument, 
     *                 'xpath' => DOMXPath, 
     *                 'cl_node' => DOMNode )
     */
    protected $dom_data = array(); 

    /**
     * Tries to get the filepointer for the SDMX-HD file
     * @param string $form 
     * @return DOMDocument
     */
    protected function getFile( $form ) {
        if ( array_key_exists( $form, $this->dom_data ) 
                && is_array( $this->dom_data[$form] ) ) {
            return $this->dom_data[$form];
        }
        $options = $this->getStorageOptions($form);
        if ( !$options instanceof I2CE_MagicDataNode ) {
            I2CE::raiseError( "Invalid SDMX-HD storage options for $form" );
            return false;
        }
        $file = null;
        if ( !$options->setIfIsSet( $file, "file" ) ) {
            I2CE::raiseError( "SDMX-HD file is not set" );
            return false;
        }
        $cl_id = null;
        if ( !$options->setIfIsSet( $cl_id, "CodeListID" ) ) {
            I2CE::raiseError( "SDMX-HD CodeListID is not set" );
            return false;
        }
        $sdmx_file = null;
        if ( !I2CE_FileSearch::isAbsolut( $file ) ) {
            $sdmx_file = I2CE::getFileSearch()->search( 'SDMXHD', $file );
        } else {
            $sdmx_file = $file;
        }
        if ( !$sdmx_file || !is_readable( $sdmx_file ) ) {
            I2CE::raiseError( "Could not find SDMX-HD $file ($sdmx_file)" );
            return false;
        }
        $doc = new DOMDocument();
        if ( !$doc->load( $sdmx_file ) ) {
            I2CE::raiseError( "Could not open $sdmx_file for reading." );
            return false;
        }
        $xpath = new DOMXPath( $doc );
        $entries = $xpath->query( "//structure:CodeList[@id='" . $cl_id . "']" );
        if ( $entries->length != 1 ) {
            I2CE::raiseError( "Could not find code list id ($cl_id) in $sdmx_file." );
            return false;
        }
        $cl_node = $entries->item(0);
        $this->dom_data[$form] = array( 'doc' => $doc, 'xpath' => $xpath,
                'cl_node' => $cl_node );
        return $this->dom_data[$form];
    }


    /**
     * Unset the data associated with the form if it exists.
     * @param string $form 
     */
    protected function closeFile( $form ) {
        if ( array_key_exists( $form, $this->dom_data ) 
                && is_array( $this->dom_data[$form] ) ) {
            $this->dom_data[$form] = false;
        }
    }
    

    /**********************************
     * 
     * Implement the abstract read methods
     *
     **********************************


    
    /**
     * Checks to see if this is writalbe
     * @return boolean
     */
    public function isWritable() {
        return false;
    }



    /**
     * Return an array of all the record ids for a given form.
     * @param string $form
     * @return array
     */
    public function getRecords( $form ) {

        $dom_data = $this->getFile( $form );

        $entries = $dom_data['xpath']->query( "./structure:Code/@value", 
                $dom_data['cl_node'] );

        $results = array();
        foreach( $entries as $entry ) {
            array_push( $results, $entry->value );
        }
        if ( $this->closeFile ) {
            $this->closeFile( $form );
        }
        return $results;
    }
    

    /**
     * Populate the member variables of the object from the database.
     * SDMX-HD will only work as a SimpleList
     * @param I2CE_SimpleList $form
     * @return boolean
     */
    public function populate( $form ) {
        $formName = $form->form();
        if ( ! $form instanceof I2CE_SimpleList ) {
            I2CE::raiseError( "SDMX-HD can't be used with anything other than SimpleLists.  Tried with: $formName" );
            return false;
        }
        $dom_data = $this->getFile( $formName );
        $entries = $dom_data['xpath']->query( "./structure:Code[@value='" 
                . $form->getId() . "']/structure:Description", 
                $dom_data['cl_node'] );
        if ( $entries->length != 1 ) {
            //not found in file so assume everything is ok
            I2CE::raiseMessage( "Failed to find " . $form->getId() 
                    . " in SDMX-HD file." );
            return true;
        }

        $name = $entries->item(0)->textContent;
        $nameField = $form->getField( "name" );
        $nameField->setFromDB( $name );

        return true;
    }


    
    

}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
